
# q-expansion of newform 230.2.e.b, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 230
weight = 2
poly_data = [1, 0, 10, 0, 19, 0, 10, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[5, 9, 18, 9, 10, 1, 1, 0], 4], [[5, -9, 18, -9, 10, -1, 1, 0], 4], [[4, 0, 11, 0, 9, 0, 1, 0], 2], [[-4, 5, 9, 18, 9, 10, 1, 1], 4], [[0, 5, -9, 18, -9, 10, -1, 1], 4], [[0, 9, 0, 29, 0, 19, 0, 2], 2]]

hecke_ring_character_values = [[47, [0, 0, 0, 0, 0, 0, 0, -1]], [51, [-1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, -1, 0, 0, 0, 0], [-1, 0, -1, 0, 0, 0, -1, -1], [1, -1, -1, -1, -1, 0, 0, -1], [1, -1, 0, 0, -1, 0, 1, 0], [0, 1, 2, -2, 0, 0, 0, 3], [-2, 1, -2, 0, 1, 0, -3, -1], [2, 1, 1, 0, 1, 0, -1, 3], [-2, 0, -1, -1, 1, 1, -1, 0], [-1, -2, -2, 1, -2, 0, 0, 1], [0, 2, -1, 1, 0, 0, 0, 0], [1, 0, -1, -1, -3, -2, 2, 0], [2, -1, -6, 0, -1, 0, 2, 1], [1, 0, 0, 0, 1, -1, 1, 0], [0, -1, 0, -2, 1, 6, 0, 5], [-1, 0, 0, 0, 0, 2, 0, 3], [2, 2, 0, 2, -2, 2, 0, 2], [3, -2, 0, 0, 0, 3, 3, 2], [2, -1, 1, -1, 0, 2, 2, -7], [-5, 3, -6, 0, 3, 0, -6, -2], [-3, 0, -3, -3, -5, 1, -1, 0], [-1, 5, 4, 0, 5, 0, -4, 4], [-1, 0, 0, 0, 4, -3, 3, 0], [-6, 3, 0, 0, -3, -8, 0, 1], [-5, 0, 0, 0, 0, 5, -5, 0], [4, -7, -9, 0, -7, 0, 3, -3], [-1, 0, 2, 2, 0, 3, -3, 0], [-6, -1, 0, 3, 1, -5, 0, 0], [-8, 2, 6, 0, 2, 0, 0, -6], [1, 0, -4, -4, -7, -4, 4, 0], [0, -2, 0, -6, 2, 6, 0, 4], [6, 2, 0, -8, -2, 2, 0, -2], [-1, 0, 5, 5, -2, 1, -1, 0], [1, 0, 8, 0, 0, 0, -1, 1], [-4, -2, 3, -3, 0, -4, -4, -6], [-5, 0, 2, 2, 5, -2, 2, 0], [9, 0, 3, 3, -1, -1, 1, 0], [-1, 4, 12, 0, 4, 0, 2, 3], [0, 8, 6, 0, 8, 0, -1, 8], [-8, 1, 0, 4, -1, -10, 0, -1], [-6, 2, 7, 0, 2, 0, 1, -4], [-1, -4, -9, 9, 0, -1, -1, -4], [4, -5, 0, 0, 0, 4, 4, -10], [5, 2, 4, -4, 0, 5, 5, 4], [11, -4, -2, 0, -4, 0, 10, 7], [-9, -2, 0, -1, 2, -3, 0, 4], [2, 0, 7, 7, 8, 8, -8, 0], [13, 0, 5, 5, -8, -7, 7, 0], [-5, 2, -4, 0, 2, 0, 4, -3], [-11, -4, 2, 0, -4, 0, -8, -15], [-12, 0, 4, 4, 4, 6, -6, 0], [12, -8, -12, 0, -8, 0, 2, 4], [-6, 6, 6, -6, 0, -6, -6, 0], [-8, 2, -5, 5, 0, -8, -8, -2], [3, -1, -9, 9, 0, 3, 3, 0], [-10, 1, 0, 10, -1, -6, 0, 5], [-8, -3, 0, -3, 3, -5, 0, 0], [1, 6, 8, -8, 0, 1, 1, 10], [-3, 0, -7, -7, -3, 2, -2, 0], [0, -4, 0, 20, 4, 2, 0, -2], [4, -10, 1, -1, 0, 4, 4, 16], [13, 2, 0, -16, -2, 10, 0, -1], [3, 5, 0, -18, -5, -2, 0, 0], [12, 6, 0, -23, -6, 3, 0, -3], [-8, 0, -6, -6, 8, 6, -6, 0], [-2, 6, 0, -2, -6, -11, 0, -3], [3, -5, 0, 0, 5, -5, 0, -13], [-2, 0, 0, 0, 6, 4, -4, 0], [3, -4, -2, 0, -4, 0, -3, -1], [4, -2, 0, 5, 2, -5, 0, -11], [-7, 4, 7, -7, 0, -7, -7, -12], [0, -10, -6, 0, -10, 0, -2, -10], [0, 0, -20, -20, 0, 2, -2, 0], [-6, -2, 14, 0, -2, 0, 2, -8], [-10, 1, 0, -10, -1, 4, 0, 15], [18, 0, 4, 4, -15, -10, 10, 0], [8, 0, 0, 4, 0, 6, 0, -2], [-8, 0, -3, -3, -1, 0, 0, 0], [-7, -16, 0, 7, 16, 11, 0, 2], [-1, -6, -9, 9, 0, -1, -1, 16], [11, -11, -6, 6, 0, 11, 11, 4], [-12, 0, 2, 2, 16, 2, -2, 0], [-2, 17, 12, -12, 0, -2, -2, 4], [-10, 12, -9, 9, 0, -10, -10, -18], [4, -4, 0, 12, 4, -1, 0, -9], [-2, -13, -7, 7, 0, -2, -2, -12], [-4, 6, 12, 0, 6, 0, -17, 2], [3, 3, -6, 6, 0, 3, 3, 28], [-8, 6, 8, 0, 6, 0, 0, -2], [3, 0, 1, 1, -8, -1, 1, 0], [-2, 5, -4, 0, 5, 0, -2, 3], [-1, 8, 10, 0, 8, 0, 4, 7], [20, 0, -16, -16, -18, 2, -2, 0], [1, 0, 0, 10, 0, 12, 0, 11], [-3, 0, 2, 2, 8, -7, 7, 0], [6, -20, -2, 2, 0, 6, 6, 6], [6, -7, 0, 18, 7, 1, 0, -12], [12, 8, 9, -9, 0, 12, 12, 12], [5, -2, -10, 10, 0, 5, 5, 12], [-1, -6, 0, 22, 6, 0, 0, -5], [4, 0, -2, -2, -8, -12, 12, 0], [5, -6, 0, -8, 6, 11, 0, 0], [1, -6, -6, 0, -6, 0, 0, -5], [-7, 5, 0, 20, -5, 0, 0, 12], [8, 0, -17, -17, 4, 8, -8, 0], [4, -3, 0, 0, 0, 4, 4, 21], [16, 9, 0, -12, -9, 6, 0, -1], [4, 2, 0, -31, -2, 5, 0, 3], [11, 7, 6, 0, 7, 0, -8, 18], [5, -3, 19, -19, 0, 5, 5, 5], [6, 0, 4, 4, 11, 7, -7, 0], [5, 6, 0, 2, -6, 4, 0, 5], [-9, 5, 0, 8, -5, -6, 0, 8], [-4, 7, 1, 0, 7, 0, -17, 3], [20, 0, -25, -25, -7, 7, -7, 0], [1, 14, 13, -13, 0, 1, 1, 4], [-1, -4, 20, -20, 0, -1, -1, 4], [-8, -8, 0, -8, 8, 2, 0, 2], [-9, -1, 0, 8, 1, -12, 0, -4], [0, -3, -30, 0, -3, 0, -15, -3], [0, 0, -4, -4, 6, -8, 8, 0], [6, -5, 9, -9, 0, 6, 6, 9], [-2, -5, 18, 0, -5, 0, 6, -7], [18, -12, 16, 0, -12, 0, 12, 6], [4, -6, 12, 0, -6, 0, 15, -2], [-4, 0, 7, 7, -10, -2, 2, 0], [16, -3, 12, -12, 0, 16, 16, 12], [2, 0, 13, 13, 9, -6, 6, 0], [16, -5, -7, 7, 0, 16, 16, 10], [-25, 9, -11, 0, 9, 0, -11, -16], [-14, 2, 0, -14, -2, -6, 0, 10], [1, -12, -16, 16, 0, 1, 1, 2], [-8, 3, 0, 14, -3, -11, 0, 0], [-6, -14, -21, 21, 0, -6, -6, -10], [-7, 7, -10, 0, 7, 0, -8, 0], [-20, 0, 8, 8, 9, -5, 5, 0], [9, 0, -11, -11, -16, 1, -1, 0], [21, -6, 0, 12, 6, 14, 0, -13], [-8, 11, 26, 0, 11, 0, -8, 3], [31, -5, 2, 0, -5, 0, 16, 26], [7, -17, 10, -10, 0, 7, 7, -3], [10, 0, 3, 3, 0, -16, 16, 0], [-4, 0, -8, -8, 10, 18, -18, 0], [20, -6, 2, 0, -6, 0, 16, 14], [-13, -3, -28, 0, -3, 0, 2, -16], [-1, 2, 17, -17, 0, -1, -1, 12], [-16, 0, 13, 13, 2, 6, -6, 0], [-4, 7, -8, 0, 7, 0, -9, 3], [-7, 10, 0, -30, -10, -16, 0, 1], [-3, 8, -26, 26, 0, -3, -3, -4], [-26, 5, 0, 0, 5, 0, -8, -21], [-17, 10, 0, -23, -10, -13, 0, 14], [1, -6, -20, 20, 0, 1, 1, 2], [12, -6, -2, 0, -6, 0, -1, 6], [-13, -15, 0, -4, 15, 14, 0, 12], [-3, 4, 26, 0, 4, 0, 14, 1], [-1, 4, 3, -3, 0, -1, -1, -34], [-16, 0, -15, -15, 2, -6, 6, 0], [2, 6, 28, -28, 0, 2, 2, 0], [-3, -12, -14, 0, -12, 0, 9, -15], [2, 9, -5, 5, 0, 2, 2, 1], [-25, 4, 0, 10, -4, -3, 0, 26], [13, -1, 0, 1, 1, 3, 0, -11], [1, 4, 0, 4, -4, 6, 0, 9], [6, -7, 6, -6, 0, 6, 6, -9], [-13, 12, 28, 0, 12, 0, 5, -1], [8, 5, 0, -29, -5, 7, 0, 4], [11, 0, 19, 19, 1, -13, 13, 0], [2, -14, 0, -6, 14, -10, 0, -26]]
